/* Dave Pallot */
/* pallot@iinet.net.au */
/* Copyright 2000 */

#ifndef SOCKET_H
#define SOCKET_H

#include <windows.h>
#include <winsock.h>

#include "InetAddress.h"

class Socket
{
public:
	enum SocketState
	{
		Init,
		Closed,
		Connected,
		Listening
	};

public:
    Socket();
    virtual ~Socket();

public:
	int initSocket();
	void close();
	int connect(const char* hostname, unsigned short port);
	int listen(unsigned short port, int backlog = SOMAXCONN);
	int accept(Socket& socket);
	unsigned int sendBuffer(const char* buffer, unsigned int size);
	unsigned int readBuffer(char* buffer, unsigned int size);

	Socket& operator >> (bool& data);
	Socket& operator >> (char& data);
	Socket& operator >> (int& data);
	Socket& operator >> (unsigned int& data);

	Socket& operator << (bool data);
	Socket& operator << (char data);
	Socket& operator << (int data);
	Socket& operator << (unsigned int data);

	static bool m_init;
	static unsigned int m_instance;


public:
	SocketState getSocketState() { return m_state; }
	const InetAddress* getInetAddr() const { return &m_inetAddr; }


public:
	inline void clearSet(fd_set& set)
	{
		FD_ZERO(&set);	
	}

	inline void addSet(fd_set& set)
	{
		FD_SET(m_socket, &set);
	}

	inline int isSet(fd_set& set)
	{
		return FD_ISSET(m_socket, &set);
	}

	inline void removeSet(fd_set& set)
	{
		FD_CLR(m_socket, &set);
	}

	inline int readSelect(fd_set& set)
	{
		return ::select(0, &set, 0, 0, 0);
	}

private:
	SocketState m_state;

	SOCKET m_socket;

	InetAddress m_inetAddr;
};

#endif //SOCKET_H